/* This file is part of the Palabos library.
 * Copyright (C) 2009, 2010 Jonas Latt
 * E-mail contact: jonas@lbmethod.org
 * The most recent release of Palabos can be downloaded at 
 * <http://www.lbmethod.org/palabos/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** \file
 * Helper functions for domain initialization -- header file.
 */
#ifndef INITIALIZATION_FUNCTIONAL_3D_H
#define INITIALIZATION_FUNCTIONAL_3D_H

#include "core/globalDefs.h"
#include "atomicBlock/dataProcessingFunctional3D.h"

namespace plb {
    
    

template<typename T1, template<class U> class Descriptor1>
class IniFreeSurfaceFlagFunctional3D : public BoxProcessingFunctional3D
{
public :
    IniFreeSurfaceFlagFunctional3D(plint iniWaterLevel_, Box3D inletPouringLiquid_);
    virtual void processGenericBlocks(Box3D domain, std::vector<AtomicBlock3D*> atomicBlocks);
   
    virtual IniFreeSurfaceFlagFunctional3D<T1,Descriptor1>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private :
    plint iniWaterLevel;
    Box3D inletPouringLiquid;
};
    
    



template<typename T, template<class U> class Descriptor>
class DefineBulkStatusFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    DefineBulkStatusFunctional3D(plint erodableWallLimit_);
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& lattice, ScalarField3D<int>& flag);
    
    virtual DefineBulkStatusFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
    plint erodableWallLimit;
        
};


template<typename T, template<class U> class Descriptor>
class IniWallFlagsFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    IniWallFlagsFunctional3D(plint erodableWallLimit_);
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& lattice, ScalarField3D<int>& flag);
    
    virtual IniWallFlagsFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
    plint erodableWallLimit;
        
};




template<typename T, template<class U> class Descriptor>
class IniWallFlagsCylinderTestFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    IniWallFlagsCylinderTestFunctional3D(plint centerX_, plint centerZ_, plint radius_);
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& lattice, ScalarField3D<int>& flag);
    
    virtual IniWallFlagsCylinderTestFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
    plint centerX, centerZ, radius;
};


template<typename T, template<class U> class Descriptor>
class IniWallFlagsDamBreakTestFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    IniWallFlagsDamBreakTestFunctional3D(plint centerX_, plint centerZ_, plint radius_);
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& lattice, ScalarField3D<int>& flag);
    
    virtual IniWallFlagsDamBreakTestFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
    plint centerX, centerZ, radius;
};


template<typename T, template<class U> class Descriptor>
class IniWallFlagRiverTestFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& lattice, ScalarField3D<int>& flag);
    
    virtual IniWallFlagRiverTestFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
};









template<typename T1, template<class U> class Descriptor1, typename T2, template<class U> class Descriptor2>
class IniDynamicsForParticlesByUsingLatticeFunctional3D : public BoxProcessingFunctional3D_LL<T1,Descriptor1,T2,Descriptor2>
{
public :
    IniDynamicsForParticlesByUsingLatticeFunctional3D(Dynamics<T1,Descriptor1>* dynamics_);
    virtual void process(Box3D domain, BlockLattice3D<T1,Descriptor1>& particles, BlockLattice3D<T2,Descriptor2>& lattice);
    virtual IniDynamicsForParticlesByUsingLatticeFunctional3D<T1,Descriptor1,T2,Descriptor2>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
     Dynamics<T1,Descriptor1>* dynamics;
};


template<typename T, template<class U> class Descriptor>
class IniDynamicsPerRiverFunctional3D : public BoxProcessingFunctional3D_LS<T,Descriptor,int>
{
public :
    IniDynamicsPerRiverFunctional3D(Dynamics<T,Descriptor>* dynamics_);
    virtual void process(Box3D domain, BlockLattice3D<T,Descriptor>& particles, ScalarField3D<int>& flagFluid);
    
    virtual IniDynamicsPerRiverFunctional3D<T,Descriptor>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
private:
     Dynamics<T,Descriptor>* dynamics;
};



}  // namespace plb

#endif  // THROMBOSIS_FUNCTIONAL_3D_H
