/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef NTENSOR_POST_PROCESSING_3D_H
#define NTENSOR_POST_PROCESSING_3D_H

#include "core/globalDefs.h"
#include "core/geometry3D.h"
#include "core/block3D.h"
#include "core/units.h"
#include "io/vtkDataOutput.h"
#include "multiBlock/multiDataField3D.h"

namespace plb {

template<typename T>
class OffLatticePostProcessing {
public:
    OffLatticePostProcessing(Group3D& fields_);
    T computeAverage(MultiNTensorField3D<T>& data);
    Array<T,3> computeAverageVect(MultiNTensorField3D<T>& data);
    void process(MultiNTensorField3D<T>& data);
private:
    Group3D& fields;
};

template<typename T, template<typename U> class Descriptor>
void offLatticeMacroscopic( Group3D& macroFields, Group3D& fields, Units3D& units,
                            T rho0=1.0, T rho0LB=1.0 );

template<typename T, template<typename U> class Descriptor>
void offLatticeForce( Group3D& macroFields, Group3D& fields, Units3D& units,
                      bool isCompressible=true, T rho0=1.0, T rho0LB=1.0 );

template<typename T, template<typename U> class Descriptor>
void writeVTK( Group3D& fields, Units3D& units, int flowType, plint iT,
               plint padding=8, T rho0=1.0, T rho0LB=1.0 );

}  // namespace plb

#endif  // NTENSOR_POST_PROCESSING_3D_H

