/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TRIANGLE_MESH_SHAPE_H
#define TRIANGLE_MESH_SHAPE_H

#include "core/globalDefs.h"
#include "core/geometry3D.h"
#include "offLattice/boundaryShapes3D.h"
#include "geometry/connectedTriangleMesh.h"
#include "boundary/boundaryProfiles.h"

namespace plb {

template< typename T, class SurfaceData >
class TriangleMeshShape3D : public BoundaryShape3D<T,SurfaceData> {
public:
    TriangleMeshShape3D (
            ConnectedTriangleMesh<T>& mesh,
            BoundaryProfileMap<T,SurfaceData> profiles );
    TriangleMeshShape3D (
            ConnectedTriangleMesh<T>& mesh );
    virtual bool isInside(Dot3D const& location) const;
    virtual bool pointOnSurface (
            Array<T,3> const& fromPoint, Array<T,3> const& direction,
            Array<T,3>& locatedPoint, T& distance,
            Array<T,3>& wallNormal, SurfaceData& surfaceData,
            OffBoundary::Type& bdType, plint& id ) const;
    virtual Array<T,3> computeContinuousNormal (
            Array<T,3> const& p, plint id, bool isAreaWeighted = false ) const;
    virtual bool intersectsSurface (
            Array<T,3> const& p1, Array<T,3> const& p2, plint& id ) const;
    virtual plint getTag(plint id) const;
    virtual bool distanceToSurface( Array<T,3> const& point,
                                    T& distance, bool& isBehind ) const;
    virtual TriangleMeshShape3D<T,SurfaceData>* clone() const;
    /// Use this clone function to provide the meshed data to this object.
    /** The arguments are:
     *  0: The voxel flags (NTensorField3D<int>),
     *  1: The hash container (AtomicContainerBlock3D),
     *  2: The boundary argument: an additional argument needed by BoundaryProfiles
     *     in order to compute the boundary condition. In dynamic walls this is for
     *     example often a MultiParticleField, used to determined the wall velocity.
     **/
    virtual TriangleMeshShape3D<T,SurfaceData>*
                clone(std::vector<AtomicBlock3D*> args) const;
private:
    ConnectedTriangleMesh<T>& mesh;
    plint bdTagID;
    BoundaryProfileMap<T,SurfaceData> profiles;
    /// Data from previous voxelization.
    NTensorField3D<int>* voxelFlags;
    /// Needed for fast access to the mesh.
    AtomicContainerBlock3D* hashContainer;
    AtomicBlock3D* boundaryArg;
};

}  // namespace plb

#endif  // TRIANGLE_MESH_SHAPE_H

