/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONNECTED_TRIANGLE_GENERATOR_H
#define CONNECTED_TRIANGLE_GENERATOR_H

#include "geometry/connectedTriangleMesh.h"

namespace plb {

namespace connected {

template<typename T>
RawConnectedTriangleMesh<T> constructSphere (
        Array<T,3> const& center, T radius, plint minNumOfTriangles );

template<typename T>
RawConnectedTriangleMesh<T> constructCylinder (
        Array<T,3> const& inletCenter, T inletRadius, T outletRadius,
        T length, plint nAxial, plint nCirc );

template<typename T>
RawConnectedTriangleMesh<T> constructCylinder (
        Array<T,3> const& inletCenter, T inletRadius, T outletRadius,
        T length, plint nAxial, plint nCirc, std::vector<Array<T,3> >& inletPoints );

template<typename T>
RawConnectedTriangleMesh<T> constructCylinder (
        Array<T,3> const& inletCenter, Array<T,3> const& axis,
        T inletRadius, T outletRadius, T length, plint nAxial, plint nCirc,
        std::vector<Array<T,3> >& inletPoints );

template<typename T>
RawConnectedTriangleMesh<T> constructCuboid (
        Array<T,3> const& lowerCorner, Array<T,3> const& upperCorner,
        Array<plint,3> const& nSegments );

template<typename T>
RawConnectedTriangleMesh<T> patchTubes(RawTriangleMesh<T>& geometryWithOpenings, plint sortDirection, std::vector<T> patchLengths);

template<typename T>
RawConnectedTriangleMesh<T> constructRectangle(T lx, T ly, plint nx, plint ny);

template<typename T>
RawConnectedTriangleMesh<T> constructStrip(std::vector<Array<T,3> > from, std::vector<Array<T,3> > to);

} // namespace connected

}  // namespace plb

#endif  // CONNECTED_TRIANGLE_GENERATOR_HH

