/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONNECTED_TRIANGLE_UTIL_H
#define CONNECTED_TRIANGLE_UTIL_H

#include "geometry/connectedTriangleMesh.h"

namespace plb {

/// Extracts a part from a connected mesh (of any kind, could be the DEF one),
/// and create a RawConnectedTriangleMesh from it.
template<typename T>
RawConnectedTriangleMesh<T> extractConnectedPart(ConnectedTriangleMesh<T>& mesh, plint whichPart);

/// Returns a vector with the positions of all the unique vertices of a ConnectedTriangleMesh.
/// All parts of the mesh are included without distinction.
template<typename T>
std::vector<Array<T,3> > getAllUniqueVertices(ConnectedTriangleMesh<T>& mesh);

template<typename T>
class MeshConnector {
public:
    MeshConnector(TriangleMesh<T>& triangleMesh_, T eps_=getEpsilon<T>(DBL));
    RawConnectedTriangleMesh<T> generateConnectedMesh() const;
private:
    void uniqueVertices();
private:
    // To unify duplicated vertices they need to be inserted in a std::set container.
    struct VertexSetNode {
        VertexSetNode(plint i_, Array<T,3> const* vertex_)
            : i(i_), vertex(vertex_)
        { }
        plint i;                  // Global index of the vertex
        Array<T,3> const* vertex; // Pointer to vertex coordinates
    };

    class VertexSetLessThan {
    public:
        VertexSetLessThan(T epsilon_)
            : epsilon(epsilon_)
        { }
        bool operator()(VertexSetNode const& node1, VertexSetNode const& node2);
    private:
        bool vertexComponentLessThan(T x, T y);
        bool vertexComponentEqual(T x, T y);
        bool vertexLessThan(Array<T,3> const& v1, Array<T,3> const& v2);
    private:
        T epsilon;
    };
    typedef std::set<VertexSetNode,VertexSetLessThan> VertexSet;
    typedef typename VertexSet::iterator VertexSetIterator;
    typedef typename VertexSet::const_iterator VertexSetConstIterator;
private:
    TriangleMesh<T>& triangleMesh;
    T eps;
    std::vector<Array<T,3> > vertices;                  // Positions of vertices. The vector
                                                        // index is the global vertex index.
    std::vector<Array<plint,3> > triangles;             // Global indices of the vertices that
                                                        // constitute the triangle. The vector
                                                        // index is the global triangle index.
    std::vector<plint> partTagging;
};

template<typename T>
void inflate(ConnectedTriangleMesh<T>& mesh, T distance);

}  // namespace plb

#endif  // CONNECTED_TRIANGLE_UTIL_H

