/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef X3D_FILE_IO_H
#define X3D_FILE_IO_H

/* X3D I/O stuff. Everything is independent of whether it's going to be used
 * on a RawTriangle, or a RawConnectedTriangleMesh, or a DEFtriangleMesh.
 */


#include "core/globalDefs.h"
#include "geometry/connectedTriangleMesh.h"

namespace plb {

/// With the two last parameters, the geometry is first shifted (by -location) and
/// then rescaled (by 1./scale), and then again shifted by (-1.,-1.,-1.).
/// If scale is less than 0, default action is taken:
/// the geometry is put into a [-1, +1] box along the longest direction.
/// The calculated location and scale are then returned to the caller.
template<typename T>
void writeX3D_json(RawConnectedTriangleMesh<T> mesh, FileName fname, Array<T,3>& location, T& scale);

/// With the two last parameters, the geometry is first shifted (by -location) and
/// then rescaled (by 1./scale), and then again shifted by (-1.,-1.,-1.).
/// If scale is less than 0, default action is taken:
/// the geometry is put into a [-1, +1] box along the longest direction.
/// The calculated location and scale are then returned to the caller.
template<typename T>
void writeX3D_blender (
        RawConnectedTriangleMesh<T> mesh, FileName fname,
        Array<T,3>& location, T& scale, bool htmlFormat=true,
        std::string propertyName="", std::string colormap="leeloo" );

template<typename T>
void x3d_transform( RawConnectedTriangleMesh<T> const& mesh,
                    Array<T,3>& location, T& scale );

template<typename T>
void to_x3d_units(RawConnectedTriangleMesh<T>& mesh, Array<T,3> location, T scale);

}  // namespace plb

#endif  // X3D_FILE_IO_H

